/* Eladine ! V1.1 
 * Autor: Laura Dietz (http://www.myselfine.de)
 *
 * Kommerzieller Vertrieb und kommerzielle Nutzung sind nicht erlaubt.
 * Aenderungen des Sourcecodes sind erlaubt, solange diese Anmerkung erhalten bleibt.
 */


/*************************************************/
/*   HIER VARIABLEN ANPASSEN!                    */

/* Der Hostname des Gatewayrechners */
server_name = "gateway"

/* Die LAN-IP Adresse des Gatewayrechners (von dem Interface an den Eladine gebunden werden soll)  */
server_ip_address = "192.168.1.1"

/* Der Port an dem der EladineClient Anfragen stellt. (Fuer den von mir kompilierten EladineClient 3031) */
eladine_server_port = "3031"

/* Eine IP-Adresse im Internet, die nur erreichbar ist, wenn man online ist (z.B. der naechste ausserhalb liegende Hop)*/
ping_test_ip = "194.64.181.2"

/* Hier alle Rechner des LANs eintragen, die rauswaehlen duerfen */
/* Einmal mit dem Namen des Rechners und einmal mit seiner IP Adresse. Die Zahlen immer schoen weiterzaehlen, ja? */
clients.!name.1 = "client1"
clients.!dot.1  = "192.168.1.2"

clients.!name.2 = "client2"
clients.!dot.2  = "192.168.1.3"

clients.!name.3 = "client3"
clients.!dot.3  = "192.168.1.4"

clients.!name.4 = "client4"
clients.!dot.4  = "192.168.1.5"

/* Anzahl der eingetragenen Rechner (die hoechste verwendete Zahl) */
clientcount = 4

/* Textdatei in der die erste Zeile die IP-Adresse beinhaltet, unter der der Gateway aus dem Internet erreichbar ist */
injoyfile = "C:\injoy\connect.txt"
/*************************************************/

SAY "servername:" server_name   /**/


/* 
 * Protokoll: <keyword> [<msg>] (Die Laenge der Sprechakte ist < 1024 chars)
 *  C "shutdown"  // darauf hin beendet sich eladine
 *  C "dialin"    // der gateway rechner waehlt sich ein (in sofern noch nicht geschehen)
 *  C "dialout"   // meldet den client ab (sind alle clients abgemeldet, so waehlt eladine aus)
 *  C "status"    // abfrage des einwahlstatus
 *  C "help"      // hinweise zu Bedienung von eladine ueber Telnet auf port 3031
 *  S "ok"       //  kommando erfolgreich ausgefuehrt
 *  S "huh"      //  kommando unbekannt
 *  S "err"      //  die anfrage ist fehlerhaft
 *  S "no"       //  die anfrage kann nicht ausgefuehrt werden
 *
 *
 */

SAY "Eladine!"


DO nr = 1 to clientcount
  clients.!d.nr = 0
END
clients.!name.0 = clientcount
clients.!dot.0 = clientcount
clients.!d.0 = clientcount

cr = "0D0A"x

CALL RxFuncAdd 'SockLoadFuncs', 'rxsock', 'SockLoadFuncs'
CALL SockLoadFuncs
CALL SockInit

CALL RxFuncAdd 'SysSleep', 'RexxUtil', 'SysSleep'
CALL RxFuncAdd 'RxIpInit','SurfRexx','RxIpInit'
CALL RxIpInit

CALL RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
CALL SysLoadFuncs

Say RxIpVersion()

dialedin = 0
myselfd = 0
twiced = 0
rotraudd = 0
eberhardd = 0


listen = SockSocket("AF_INET","SOCK_STREAM",0)
IF listen < 0 THEN CALL error "could not create welcome socket."

servaddress.!family = "AF_INET"
servaddress.!port = eladine_server_port
servaddress.!addr = server_ip_address
ret = SockBind(listen,"servaddress.!")
IF ret < 0 THEN CALL error "could not bind welcome socket. Make sure no other application uses port "eladine_server_port" and wait some minutes until you try again."

ret = SockListen(listen, 1024)
IF ret < 0 THEN CALL error "SockListen"

DO FOREVER
  CALL serve
END

EXIT    


SERVE:
  r.!0 = 1
  r.!1 = listen
  ret = SockSelect("r.!", , ,1)
  IF ret < 0 THEN CALL error "SockSelect"
  /* ret == 0 bei timeout */
  IF ret > 0 THEN DO
    IF r.!0 > 0 THEN DO
    
      /* VERBINDUNGSANFRAGE */
      socket = SockAccept(listen, "addr.!")
      
      /* namen auslesen */
      /*ret = SockGetHostByAddr(addr.!addr, "host.!") - dauert zu lange*/
      clientdot = addr.!addr
      CALL hostname clientdot
      clientname = RESULT

      DROP addr.!
      DROP host.!
      SAY "accept"
      ret = SockSend(socket, "Eladine@"server_name"> ")
      
     /* read */
     chars = SockRecv(socket, msg, 1024)
     SAY "received"
     IF chars >= 1024 THEN DO
       SAY "msg too long: " msg
       ret = SockSend(socket, "err msg too long. max 1024 chars accepted")
       ret = SockClose(socket)
     END
     ELSE DO 
      IF chars <3 THEN DO
        SAY "msg too short." msg
        ret = SockSend(socket, "err msg too short.")
        ret = SockClose(socket)
      END
     
      ELSE DO 
      /* hier msg parsen und antworten!!! */
      SAY "-" substr(msg,1, LENGTH(msg)-2) "- von" clientname
      
      SELECT
        /* shutdown */
        WHEN substr(msg, 1, LENGTH("shutdown")) = "shutdown" THEN DO
          SAY "shutdown"
          ret = SockSend(socket, "ok shutting down")
          ret = SockClose(socket)
          ret = SockShutDown(listen, 2)
          ret = SockClose(listen)
          EXIT
        END

        /* dialin */
        WHEN substr(msg, 1, LENGTH("dialin")) = "dialin" THEN DO
          SAY "dialin"
          DO nr = 1 to clientcount
            IF clients.!dot.nr == clientdot THEN DO
              clients.!d.nr = 1
            END
          END
         
          SAY "-- testcall --"
          CALL ANYONE_D
          SAY "call anyone_d " RESULT
          
          IF ANYONE_D() == 1 THEN DO
            start "dialin.cmd"
            dialedin = 1
            ret = SockSend(socket, "ok dialing in")
          END
          ELSE DO                
            ret = SockSend(socket, "err not allowed")
          END
          
          ret = SockClose(socket)
        END

        /* dialout */
        WHEN substr(msg, 1, LENGTH("dialout")) = "dialout" THEN DO
          /* wer will sich auswaehlen? */
          DO nr = 1 to clientcount
            IF clients.!name.nr == clientname THEN clients.!d.nr = 0
          END
          
          IF ANYONE_D() == 0 THEN DO
          
          /* keiner will online sein */
          
            start "dialout.cmd"
            dialedin = 0

            ret = SockSend(socket, "ok dialing out")
          END
          ELSE DO
          /* einer is noch am surfen */
            ret = SockSend(socket, "ok but staying online")
          END
          
          ret = SockClose(socket)
        END
        
        
        /* status */
        WHEN substr(msg, 1, LENGTH("status")) = "status" THEN DO
          ret = SockSend(socket, "ok status information follows..."cr)
          IF dialedin == 1 THEN ret = SockSend(socket, "assumed: dialed in"cr)
          ELSE ret = SockSend(socket, "assumed: NOT dialed in"cr)

          CALL ping
          online = RESULT
          IF online == 1 THEN ret = SockSend(socket, "actually: online"cr)
          ELSE ret = SockSend(socket, "actually: offline"cr)

          ret = SockSend(socket, "computers that want to be online: (1=yes, 0=no)"cr)
          DO nr = 1 to clientcount
            ret = SockSend(socket, clients.!name.nr": "clients.!d.nr cr)
          END

          ret = SockSend(socket, "-----" cr)  
          
          /* liest ip-adresse und gateway aus injoys connect.txt aus */
          ret = SockSend(socket, "IP-adress: "INJOY_LINE() cr)
          ret = SockSend(socket, "-----" cr)  
          
          /* wieviele mails sind gequeued? */
          call SysFileTree "C:\mptn\etc\mail\forward\*.fwd","mails.","F"
          ret = SockSend(socket, mails.0 " E-Mails queued" cr)
          ret = SockSend(socket, "-----" cr)  
          

          ret = SockClose(socket)
        END

        /* help */
        WHEN substr(msg, 1, LENGTH("help")) = "help" THEN DO
          ret = SockSend(socket,"ok the following commands are allowed..."cr)
          ret = SockSend(socket,"dialin     goes online and marks you to be online"cr)
          ret = SockSend(socket,"dialout    goes offline, if you are the last one"cr)
          ret = SockSend(socket,"status     display status information"cr)
          ret = SockSend(socket,"help       this information"cr)
          ret = SockSend(socket,"shutdown   do NOT use! (shuts this service down)"cr)
          ret = SockSend(socket,"-------")

          ret = SockClose(socket)
        END
          
        /* unknown */
        OTHERWISE DO
          ret = SockSend(socket, "huh ?")
          SAY "huh?"  
          ret = SockClose(socket)
          SAY "closed"
        END
      END
     END
     END
    END
  END
  DROP msg
  DROP chars
RETURN  


INJOY_LINE: 
  /* injoyfile = "C:\injoy\connect.txt" */
  ret = STREAM(injoyfile, 'C', "OPEN")
  templine = LINEIN(injoyfile)
  ret = STREAM(injoyfile, 'C', "CLOSE")
  RETURN templine

PING:
  connect = RxIpConnectWait(ping_test_ip, 80, 5)
  IF connect > 0 THEN DO
    online = 1
    ret = RxIpClose(connect)
  END
  ELSE DO
    online = 0
  END
RETURN online


ANYONE_D:
  decide_dialout = 0
  DO nr = 1 to clientcount
    decide_dialout = BITOR(decide_dialout, clients.!d.nr)
  END
RETURN decide_dialout



ERROR: /* ARG(1) = msg */
  SAY "ERROR:" ARG(1)
  EXIT



HOSTNAME: /* ARG(1) = dotted_ip_adress */
  DO nr = 1 to clientcount
    IF ARG(1)==clients.!dot.nr THEN RETURN clients.!name.nr
  END
RETURN "unknown"
